import random
from datetime import datetime
from typing import Any, Dict, List

from pytz import timezone

from sktmls.models import MLSRuleModel, MLSModelError

TZ = timezone("Asia/Seoul")
SEED_FORMAT = {"hourly": "%Y%m%d%H", "daily": "%Y%m%d", "weekly": "%Y%W", "monthly": "%Y%m", "yearly": "%Y"}


class MbrInterestAllBaselineModel(MLSRuleModel):
    """
    MLS 모델 레지스트리에 등록되는 Rule 기반 클래스입니다.

    멤버십 이종 카테고리 카드 제공에 특화된 모델입니다.
    """

    def __init__(
        self,
        model_name: str,
        model_version: str,
        features: List[str],
        categories: List[str],
        item_type: str,
        num_pick: int = 1,
        random_cycle: str = "daily",
    ):
        """
        ## Args

        - model_name: (str) 모델 이름
        - model_version: (str) 모델 버전
        - features: (list(str)) 피쳐 리스트 (필수 피쳐: `user_id`, `prefer_mbr_category`)
        - categories: (list(str)) 전체 카테고리 리스트. 필수 피쳐 이후 피쳐 순서와 동일해야 합니다.
        - item_type: (str) 결과의 `type`에 표기할 값
        - num_pick: (optional) (int) 랜덤 추출할 갯수 (기본값: 1)
        - random_cycle: (optional) (str) 랜덤 시드 초기화 주기 (`hourly`|`daily`|`weekly`|`monthly`|`yearly`) (기본값: `daily`)

        ## Example

        ```python
        my_model_v1 = MbrInterestAllBaselineModel(
            model_name="my_model",
            model_version="v1",
            features=["user_id", "prefer_mbr_category", "available_benefit_activity", "available_benefit_asianfood", "available_benefit_beauty"],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            item_type="mbr_interest_all",
            num_pick=14,
            random_cycle="daily"
        )

        result = my_model_v1.predict(["1234567890", ["mbr_beauty"], ["a1", "a2", "a3", "a4"], None, ["b1", "b2", "3", "4"]])
        ```
        """
        assert (
            isinstance(features, list) and len(features) > 0 and features[0] == "user_id"
        ), "`features`가 유효하지 않습니다. `features`의 첫 값은 반드시 `user_id`이어야 합니다."
        assert isinstance(categories, list) and len(categories) + 2 == len(
            features
        ), "전체 카테고리 리스트 길이가 필수 피쳐를 제외한 피쳐의 수와 다릅니다."
        assert isinstance(num_pick, int) and num_pick > 0, "`num_pick`이 유효하지 않습니다. 반드시 1 이상의 값을 가져야 합니다."
        assert random_cycle in SEED_FORMAT, "허용되지 않는 `random_cycle`값입니다."

        super().__init__(model_name, model_version, features)

        self.categories = categories
        self.item_type = item_type
        self.num_pick = num_pick
        self.random_cycle = random_cycle

    def predict(self, x: List[Any], **kwargs) -> Dict[str, Any]:
        for i, f in enumerate(x[1:]):
            if f is not None and not isinstance(f, list):
                raise MLSModelError(f"MbrInterestAllBaselineModel: {self.features[i + 1]}이 리스트 형식으로 전달되지 않았습니다.")

        user_id = x[0]
        prefer_mbr_category = x[1] or []
        available_benefit = {
            category: x[i + 2]
            for i, category in enumerate(self.categories)
            if isinstance(x[i + 2], list) and len(x[i + 2]) >= 1
        }

        random.seed(f"{user_id}{self.item_type}{datetime.now(TZ).strftime(SEED_FORMAT[self.random_cycle])}")

        if not available_benefit:
            return {"items": []}

        candidates = []
        for category, benefits in available_benefit.items():
            for benefit in benefits:
                candidates.append(
                    {
                        "id": benefit,
                        "name": "#",
                        "type": self.item_type,
                        "props": {
                            "score": str(round(random.random() + (1 if category in prefer_mbr_category else 0), 4)),
                            "context_id": "context_preference"
                            if category in prefer_mbr_category
                            else "context_default",
                            "context_nm": "고객선호" if category in prefer_mbr_category else "#",
                        },
                    }
                )

        num_pick = min(self.num_pick, len(candidates))
        top_candidates = sorted(candidates, key=lambda k: float(k["props"]["score"]), reverse=True)[:num_pick]

        return {"items": top_candidates}



from sktmls.models.contrib import RandomScoreModel, PeriodicRandomScoreModel


class TestRandomScoreModel:
    def test_000_random_score(self):
        rsm = RandomScoreModel(
            model_name="test_model", model_version="test_version", item_id="1", item_name="n", item_type="t"
        )

        results = rsm.predict(None)["items"][0]
        assert results["id"] == "1"
        assert results["name"] == "n"
        assert results["type"] == "t"
        assert 0 <= float(results["props"]["score"]) <= 1


class TestPeriodicRandomScoreModel:
    def test_000_periodic_random_score(self):
        rpm = PeriodicRandomScoreModel(
            model_name="test_model",
            model_version="test_version",
            item_id="1",
            item_name="n",
            item_type="t",
            random_cycle="daily",
        )

        result1 = rpm.predict("1234567890")
        result2 = rpm.predict("1234567890")
        assert result1 == result2



from functools import reduce

import numpy as np


class LogicProcessor:
    """
    상품 선택 로직 등을 프로세스하는 클래스입니다.

    아래 operations를 참조하여 json 형식의 로직을 구현하면 됩니다.
    자세한 사용법에 대한 문서를 추가 예정입니다.
    """

    def __init__(self):
        self.operations = {
            "==": self._soft_equals,
            "===": self._hard_equals,
            "!=": lambda a, b: not self._soft_equals(a, b),
            "!==": lambda a, b: not self._hard_equals(a, b),
            ">": lambda a, b: self._less(b, a),
            ">=": lambda a, b: self._less(b, a) or self._soft_equals(a, b),
            "<": self._less,
            "<=": self._less_or_equal,
            "!": lambda a: not a,
            "!!": bool,
            "%": lambda a, b: a % b,
            "and": lambda *args: reduce(lambda total, arg: total and arg, args, True),
            "or": lambda *args: reduce(lambda total, arg: total or arg, args, False),
            "?:": lambda a, b, c: b if a else c,
            "if": self._if,
            "if_min": self._if_min,
            "if_max": self._if_max,
            "if_closest": self._if_closest,
            "if_closest_upper": self._if_closest_upper,
            "if_closest_lower": self._if_closest_lower,
            "in": lambda a, b: a in b if "__contains__" in dir(b) else False,
            "cat": lambda *args: "".join(str(arg) for arg in args),
            "+": self._plus,
            "*": lambda *args: reduce(lambda total, arg: total * float(arg), args, 1),
            "-": self._minus,
            "/": lambda a, b=None: a if b is None else float(a) / float(b),
            "min": lambda *args: min(args),
            "max": lambda *args: max(args),
            "merge": self._merge,
            "count": lambda *args: sum(1 if a else 0 for a in args),
            "abs": lambda a: abs(a),
            "closest": self._closest,
            "closest_upper": self._closest_upper,
            "closest_lower": self._closest_lower,
            "return": lambda *args: args,
        }

    def _if(self, *args):
        for i in range(0, len(args) - 1, 2):
            if args[i]:
                return args[i + 1]
        if len(args) % 2:
            return args[-1]
        else:
            return None

    def _if_min(self, *args):
        min_arg = np.argmin([args[i] for i in range(0, len(args) - 1, 2)])
        return args[min_arg * 2 + 1]

    def _if_max(self, *args):
        max_arg = np.argmax([args[i] for i in range(0, len(args) - 1, 2)])
        return args[max_arg * 2 + 1]

    def _if_closest(self, arg, *comps):
        args = list(comps)
        for i in range(0, len(args) - 1, 2):
            args[i] = abs(arg - args[i])
        return self._if_min(*args)

    def _if_closest_upper(self, arg, *comps):
        args = []
        for i in range(0, len(comps) - 1, 2):
            if comps[i] > arg:
                args.append(comps[i])
                args.append(comps[i + 1])
        if not args:
            return None
        return self._if_closest(arg, *args)

    def _if_closest_lower(self, arg, *comps):
        args = []
        for i in range(0, len(comps) - 1, 2):
            if comps[i] < arg:
                args.append(comps[i])
                args.append(comps[i + 1])
        if not args:
            return None
        return self._if_closest(arg, *args)

    def _soft_equals(self, a, b):
        if isinstance(a, str) or isinstance(b, str):
            return str(a) == str(b)
        if isinstance(a, bool) or isinstance(b, bool):
            return bool(a) is bool(b)
        return a == b

    def _hard_equals(self, a, b):
        if type(a) != type(b):
            return False
        return a == b

    def _less(self, a, b, *args):
        types = set([type(a), type(b)])
        if float in types or int in types:
            try:
                a, b = float(a), float(b)
            except TypeError:
                return False
        return a < b and (not args or self._less(b, *args))

    def _less_or_equal(self, a, b, *args):
        return (self._less(a, b) or self._soft_equals(a, b)) and (not args or self._less_or_equal(b, *args))

    def _to_numeric(self, arg):
        if isinstance(arg, str):
            if "." in arg:
                return float(arg)
            else:
                return int(arg)
        return arg

    def _plus(self, *args):
        return sum(self._to_numeric(arg) for arg in args)

    def _minus(self, *args):
        if len(args) == 1:
            return -self._to_numeric(args[0])
        return self._to_numeric(args[0]) - self._to_numeric(args[1])

    def _closest(self, arg, *comps):
        closest = comps[0]
        min_diff = abs(arg - comps[0])
        for i in range(1, len(comps)):
            diff = abs(arg - comps[i])
            if min_diff > diff:
                min_diff = diff
                closest = comps[i]
        return closest

    def _closest_upper(self, arg, *comps):
        args = [c for c in comps if c > arg]
        if not args:
            return None
        return self._closest(arg, *args)

    def _closest_lower(self, arg, *comps):
        args = [c for c in comps if c < arg]
        if not args:
            return None
        return self._closest(arg, *args)

    def _merge(self, *args):
        ret = []
        for arg in args:
            if isinstance(arg, list) or isinstance(arg, tuple):
                ret += list(arg)
            else:
                ret.append(arg)
        return ret

    def _get_var(self, data, var_name, not_found=None):
        try:
            for key in str(var_name).split("."):
                try:
                    data = data[key]
                except TypeError:
                    data = data[int(key)]
        except (KeyError, TypeError, ValueError):
            return not_found
        else:
            return data

    def _missing(self, data, *args):
        not_found = object()
        if args and isinstance(args[0], list):
            args = args[0]
        ret = []
        for arg in args:
            if self._get_var(data, arg, not_found) is not_found:
                ret.append(arg)
        return ret

    def _missing_some(self, data, min_required, args):
        if min_required < 1:
            return []
        found = 0
        not_found = object()
        ret = []
        for arg in args:
            if self._get_var(data, arg, not_found) is not_found:
                ret.append(arg)
            else:
                found += 1
                if found >= min_required:
                    return []
        return ret

    def apply(self, tests, data=None):
        """
        ## Args

        - tests: (dict) 테스트 할 json 형식의 로직
        - data: (optional) (dict) 로직에서 참조할 데이터 dict (기본값: None)
        """
        if tests is None or not isinstance(tests, dict):
            return tests

        operator = list(tests.keys())[0]
        data = data or {}
        values = tests[operator]

        if not isinstance(values, list) and not isinstance(values, tuple):
            values = [values]

        values = [self.apply(val, data) for val in values]

        if operator == "var":
            return self._get_var(data, *values)
        if operator == "missing":
            return self._missing(data, *values)
        if operator == "missing_some":
            return self._missing_some(data, *values)

        if operator not in self.operations:
            raise ValueError("없는 operator입니다. %s" % operator)

        return self.operations[operator](*values)



import random
from collections import OrderedDict
from datetime import datetime
from functools import reduce
from typing import Any, Dict, List

from pytz import timezone

from sktmls import MLSRuntimeENV
from sktmls.apis import MLSProfileAPIClient
from sktmls.models import MLSRuleModel, MLSModelError

TZ = timezone("Asia/Seoul")
SEED_FORMAT = {
    "hourly": "%Y%m%d%H",
    "daily": "%Y%m%d",
    "weekly": "%Y%W",
    "monthly": "%Y%m",
    "yearly": "%Y",
    "none": "%c%f",
}


class MbrSimilarBenefitModel(MLSRuleModel):
    """
    MLS 모델 레지스트리에 등록되는 Rule 기반 클래스입니다.

    멤버십 유사 혜택 추천에 특화된 모델입니다.
    """

    def __init__(
        self,
        model_name: str,
        model_version: str,
        features: List[str],
        categories: List[str],
        item_type: str,
        contexts: List[Dict[str, Any]] = [],
        num_pick: int = 1,
        random_cycle: str = "daily",
    ):
        """
        ## Args

        - model_name: (str) 모델 이름
        - model_version: (str) 모델 버전
        - features: (list(str)) 피쳐 리스트 (필수 피쳐: `user_id`, `*_benefit`)
        - categories: (list(str)) 전체 카테고리 리스트. 필수 피쳐 이후 피쳐 순서와 동일해야 합니다.
        - item_type: (str) 결과의 `type`에 표기할 값
        - contexts: (list(dict)) context_id 생성을 위한 정보를 담은 리스트. `*_benefit` 피쳐와 순서가 동일해야 합니다. (기본값: [])
        - num_pick: (optional) (int) 랜덤 추출할 갯수 (기본값: 1)
        - random_cycle: (optional) (str) 랜덤 시드 초기화 주기 (`hourly`|`daily`|`weekly`|`monthly`|`yearly`|`none`) (기본값: `daily`)
            - `none`의 경우 매 호출마다 초기화합니다.

        ## Example

        ```python
        my_model_v1 = MbrSimilarBenefitModel(
            model_name="my_model",
            model_version="v1",
            features=["user_id", "use_benefit", "like_benefit", "available_benefit_activity", "available_benefit_asianfood", "available_benefit_beauty"],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            item_type="mbr_similar_benefit",
            contexts=[
                {"act": "use", "context_id": 35},
                {"act": "like", "context_id": 36},
            ],
            num_pick=14,
            random_cycle="daily"
        )

        result = my_model_v1.predict(["1234567890", None, ["1", "5", "6"], ["a1", "a2", "a3", "a4"], None, ["b1", "b2", "3", "4"]])
        ```
        """
        assert (
            isinstance(features, list) and len(features) > 0 and features[0] == "user_id"
        ), "`features`가 유효하지 않습니다. `features`의 첫 값은 반드시 `user_id`이어야 합니다."
        assert isinstance(contexts, list), "`contexts`은 반드시 리스트 타입이어야 합니다."
        assert isinstance(categories, list) and len(categories) + len(contexts) + 1 == len(
            features
        ), "전체 카테고리 리스트 길이가 필수 피쳐를 제외한 피쳐의 수와 다릅니다."
        assert isinstance(num_pick, int) and num_pick > 0, "`num_pick`이 유효하지 않습니다. 반드시 1 이상의 값을 가져야 합니다."
        assert random_cycle in SEED_FORMAT, "허용되지 않는 `random_cycle`값입니다."

        super().__init__(model_name, model_version, features)

        self.categories = categories
        self.item_type = item_type
        self.contexts = contexts
        self.num_pick = num_pick
        self.random_cycle = random_cycle

    def predict(self, x: List[Any], **kwargs) -> Dict[str, Any]:
        for i, f in enumerate(x[1:]):
            if f is not None and not isinstance(f, list):
                raise MLSModelError(f"MbrSimilarBenefitModel: {self.features[i + 1]}이 리스트 형식으로 전달되지 않았습니다.")

        pf_client = kwargs.get("pf_client") or MLSProfileAPIClient(runtime_env=MLSRuntimeENV.MMS)

        user_id = x[0]

        benefit_map = {}
        for i, context in enumerate(self.contexts):
            if x[i + 1]:
                benefit_map[context["act"]] = {"benefit": x[i + 1], "context_id": context["context_id"]}

        if not benefit_map:
            return {"items": []}

        num_acts = len(self.contexts)
        available_benefit = {
            category: x[i + num_acts + 1]
            for i, category in enumerate(self.categories)
            if isinstance(x[i + num_acts + 1], list) and len(x[i + num_acts + 1]) >= 1
        }

        random.seed(f"{user_id}{self.item_type}{datetime.now(TZ).strftime(SEED_FORMAT[self.random_cycle])}")

        source_benefit_context = []
        for benefit_info in benefit_map.values():
            for benefit in benefit_info["benefit"]:
                source_benefit_context.append((benefit, benefit_info["context_id"]))
        source_benefit_id, source_context_id = random.choice(source_benefit_context)

        item_profile = pf_client.get_item_profile(
            profile_id="tmembership_benefit", item_id=source_benefit_id, keys=["similar_items"]
        )

        if (
            None in item_profile.values()
            or not isinstance(item_profile["similar_items"], list)
            or not item_profile["similar_items"]
        ):
            return {"items": []}
        target_benefit_list = item_profile["similar_items"]

        available_benefit_list = reduce(lambda a, b: a + b, available_benefit.values(), [])
        valid_target_benefit_list = list(
            OrderedDict.fromkeys(filter(lambda x: x in available_benefit_list, target_benefit_list))
        )

        candidates = [
            {
                "id": benefit,
                "name": "#",
                "type": self.item_type,
                "props": {
                    "context_id": source_context_id,
                    "benefit_id": source_benefit_id,
                },
            }
            for benefit in valid_target_benefit_list
        ]

        num_pick = min(self.num_pick, len(candidates))
        top_candidates = candidates[:num_pick]

        return {"items": top_candidates}



import random
import string
from typing import List

from sktmls import MLSClient, MLSENV, MLSRuntimeENV, MLSResponse, MLSClientError

MLS_CLIENTS_API_URL = "/api/v1/clients"


class Client:
    """
    MLS 클라이언트 클래스입니다. `Client`는 MLS 연동에 필요한 클라이언트 정보를 가지는 클래스이며 `sktmls.MLSClient`와는 별개의 개념입니다.
    """

    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) 클라이언트 고유 ID
            - name: (str) 클라이언트 이름
            - apikey: (str) API Key
            - user: (str) 클라이언트 소유 계정명

        ## Returns
        `sktmls.clients.Client`
        """
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.apikey = kwargs.get("apikey")
        self.user = kwargs.get("user")

    def get(self):
        return self.__dict__

    def reset(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class ClientClient(MLSClient):
    """
    MLS 클라이언트 관련 기능들을 제공하는 클라이언트입니다.
    """

    def __init__(
        self,
        env: MLSENV = None,
        runtime_env: MLSRuntimeENV = None,
        username: str = None,
        password: str = None,
    ):
        """
        ## Args

        - env: (`sktmls.MLSENV`) 접근할 MLS 환경 (`sktmls.MLSENV.DEV`|`sktmls.MLSENV.STG`|`sktmls.MLSENV.PRD`) (기본값: `sktmls.MLSENV.STG`)
        - runtime_env: (`sktmls.MLSRuntimeENV`) 클라이언트가 실행되는 환경 (`sktmls.MLSRuntimeENV.YE`|`sktmls.MLSRuntimeENV.EDD`|`sktmls.MLSRuntimeENV.LOCAL`) (기본값: `sktmls.MLSRuntimeENV.LOCAL`)
        - username: (str) MLS 계정명 (기본값: $MLS_USERNAME)
        - password: (str) MLS 계정 비밀번호 (기본값: $MLS_PASSWORD)

        아래의 환경 변수가 정의된 경우 해당 파라미터를 생략 가능합니다.

        - $MLS_ENV: env
        - $MLS_RUNTIME_ENV: runtime_env
        - $MLS_USERNAME: username
        - $MLS_PASSWORD: password

        ## Returns
        `sktmls.clients.ClientClient`

        ## Example

        ```python
        client_client = ClientClient(env=MLSENV.STG, runtime_env=MLSRuntimeENV.YE, username="mls_account", password="mls_password")
        ```
        """
        super().__init__(env=env, runtime_env=runtime_env, username=username, password=password)

    def create_client(self, name: str, apikey: str = None) -> Client:
        """
        클라이언트를 생성합니다.

        ## Args

        - name: (str) 클라이언트 이름
        - apikey: (optional) (str) 길이 40의 API Key. 전달되지 않을 경우 자동 생성

        ## Returns
        `sktmls.clients.Client`

        - id: (int) 클라이언트 고유 ID
        - name: (str) 클라이언트 이름
        - apikey: (str) API Key
        - user: (str) 클라이언트 소유 계정명

        ## Example

        ```python
        client = client_client.create_client(name="my_mls_client")
        ```
        """
        if apikey is not None:
            assert len(apikey) == 40, "apikey의 길이는 반드시 40이어야 합니다."
        else:
            apikey = "".join(random.choices(string.ascii_uppercase + string.digits, k=40))

        data = {"name": name, "apikey": apikey}

        return Client(**self._request(method="POST", url=MLS_CLIENTS_API_URL, data=data).results)

    def list_clients(self, **kwargs) -> List[Client]:
        """
        클라이언트 리스트를 가져옵니다.

        ## Args

        - kwargs: (optional) (dict) 필터 조건
            - id: (int) 클라이언트 고유 ID
            - name: (str) 클라이언트 이름
            - query: (str) 검색 문자
            - page: (int) 페이지 번호

        ## Returns
        list(`sktmls.clients.Client`)

        - id: (int) 클라이언트 고유 ID
        - name: (str) 클라이언트 이름
        - apikey: (str) API Key
        - user: (str) 클라이언트 소유 계정명

        ## Example

        ```python
        all_my_clients = client_client.list_clients()
        ```
        """
        response = self._request(method="GET", url=f"{MLS_CLIENTS_API_URL}", params=kwargs).results
        return [Client(**client) for client in response]

    def get_client(self, id: int = None, name: str = None) -> Client:
        """
        클라이언트 정보를 가져옵니다.

        ## Args: `id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - id: (int) 클라이언트 고유 ID
        - name: (str) 클라이언트 이름

        ## Returns
        `sktmls.clients.Client`

        - id: (int) 클라이언트 고유 ID
        - name: (str) 클라이언트 이름
        - apikey: (str) API Key
        - user: (str) 클라이언트 소유 계정명

        ## Example

        ```python
        client_by_id = client_client.get_client(id=3)
        client_by_name = client_client.get_client(name="my_client")
        ```
        """
        assert id or name, "`id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        clients = self.list_clients(id=id, name=name)
        if len(clients) != 1:
            raise MLSClientError(code=404, msg="클라이언트가 없거나 여러개 존재합니다.")
        return clients[0]

    def delete_client(self, client: Client) -> MLSResponse:
        """
        클라이언트를 삭제합니다.

        ## Args

        - client: (`sktmls.clients.Client`) 클라이언트 객체

        ## Returns
        `sktmls.MLSResponse`

        ## Example

        ```python
        client_client.delete_client(my_client)
        ```
        """
        assert type(client) == Client

        return self._request(method="DELETE", url=f"{MLS_CLIENTS_API_URL}/{client.id}")



import pytest

from sktmls.models.contrib import InfoDefectRuleModel


class TestInfoDefectRuleModel:
    @pytest.fixture(scope="class")
    def param(self):
        return {"model": InfoDefectRuleModel("test_model", "test_version", ["svc_scrb_period", "eqp_chg_period"])}

    def test_000_info_defect_return(self, param):
        assert param.get("model").predict([10, 0]) == {
            "items": [{"id": "INF0000004", "name": "초기 불량 대처 방법 안내", "props": {}, "type": "info_defect"}]
        }

    def test_001_info_defect_return(self, param):
        assert param.get("model").predict([50, 15]) == {
            "items": [{"id": "INF0000004", "name": "초기 불량 대처 방법 안내", "props": {}, "type": "info_defect"}]
        }

    def test_002_info_defect_return(self, param):
        assert param.get("model").predict([10, 15]) == {
            "items": [{"id": "INF0000004", "name": "초기 불량 대처 방법 안내", "props": {}, "type": "info_defect"}]
        }

    def test_003_info_defect_return(self, param):
        assert param.get("model").predict([50, 0]) == {"items": []}



from enum import Enum
from typing import List


class MLSENV(Enum):
    DEV = "dev"
    STG = "stg"
    PRD = "prd"
    LOCAL = "local"

    @classmethod
    def list_items(cls) -> List["MLSENV"]:
        """
        모든 MLS 환경을 리스트로 반환합니다.
        """
        return [t for t in cls]

    @classmethod
    def list_values(cls) -> List[str]:
        """
        모든 MLS 환경 이름을 리스트로 반환합니다.
        """
        return [t.value for t in cls]


class MLSRuntimeENV(Enum):
    YE = "YE"
    EDD = "EDD"
    MMS = "MMS"
    LOCAL = "LOCAL"

    @classmethod
    def list_items(cls) -> List["MLSRuntimeENV"]:
        """
        모든 MLSRuntimeENV 환경을 리스트로 반환합니다.
        """
        return [t for t in cls]

    @classmethod
    def list_values(cls) -> List[str]:
        """
        모든 MLSRuntimeENV 환경 이름을 리스트로 반환합니다.
        """
        return [t.value for t in cls]



from typing import List, Dict, Any, Callable

from functools import partial

from sktmls.filters import Filter
from sktmls.utils import LogicProcessor

FILTER_OPERATORS = ["==", "!-", "<=", ">=", "<", ">"]


class FilterUtil:
    """
    필터 로직관련 클래스 입니다.

    ## Example

    ```python
    filter = filter_client.get_filter(id=10)

    filter_util = FilterUtil()
    filter_func = filter_util.generate_filter_applier_fn(filter)

    data = {"key1": "good1", "key2": 3, "mbr_discount_amt_cum_bakery": "N"} # (필터를 적용할 Key-Value 값)
    filter_func(data)
    ```
    """

    def __init__(self):
        self.logic_processor = LogicProcessor()

    def apply(self, data: Dict[str, Any], conditions: List[dict]) -> bool:
        """
        필터 로직 적용 함수

        ## Args
        - data: (dict) 필터를 적용할 Key-Value 값
        - conditions: (list(dict)) 필터 조건 리스트

        ## Returns
        bool
            - True : 필터 로직 통과
            - False : 필터 로직 걸림
        """
        for filter_dict in conditions:
            key_value = data.get(filter_dict.get("key"))

            if not key_value:
                continue

            operator = filter_dict.get("operator")
            operator_value = filter_dict.get("value")

            if self.logic_processor.apply({operator: [key_value, operator_value]}):
                return False

        return True

    def generate_filter_applier_fn(self, filter: Filter) -> Callable[[dict], bool]:
        """
        필터 로직을 적용하는 함수를 리턴하는 함수 (filter 디펜던시)

        ## Args
        - filter: (`sktmls.filters.Filter`) 필터 객체

        ## Returns
        필터 로직 적용 함수
        """
        assert type(filter) == Filter, "`filter`는 `sktmls.filters.Filter`객체여야 합니다."
        conditions = filter.conditions

        for filter_dict in conditions:
            if filter_dict.get("operator") not in FILTER_OPERATORS:
                raise ValueError("없는 operator입니다. %s" % filter_dict.get("operator"))

        return partial(self.apply, conditions=conditions)
